import {
  ArgumentInvalidException,
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface WaypointProps {
  position: number;
  lon: number;
  lat: number;
}

export class Waypoint extends ValueObject<WaypointProps> {
  get position(): number {
    return this.props.position;
  }

  get lon(): number {
    return this.props.lon;
  }

  get lat(): number {
    return this.props.lat;
  }

  protected validate(props: WaypointProps): void {
    if (props.position < 0)
      throw new ArgumentInvalidException(
        'position must be greater than or equal to 0',
      );
    if (props.lon > 180 || props.lon < -180)
      throw new ArgumentOutOfRangeException('lon must be between -180 and 180');
    if (props.lat > 90 || props.lat < -90)
      throw new ArgumentOutOfRangeException('lat must be between -90 and 90');
  }
}
