import { AdRepositoryPort } from '@modules/ad/core/application/ports/ad.repository.port';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { PassengerOrientedSelector } from '@modules/ad/core/application/queries/match/selector/passenger-oriented.selector';
import {
  AlgorithmType,
  Candidate,
} from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency } from '@modules/ad/core/domain/ad.types';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const matchQuery = new MatchQuery({
  algorithmType: AlgorithmType.PASSENGER_ORIENTED,
  driver: true,
  passenger: true,
  frequency: Frequency.PUNCTUAL,
  fromDate: '2023-06-21',
  toDate: '2023-06-21',
  useAzimuth: true,
  azimuthMargin: 10,
  useProportion: true,
  proportion: 0.3,
  schedule: [
    {
      day: 3,
      time: '07:05',
      margin: 900,
    },
  ],
  strict: false,
  waypoints: [originWaypoint, destinationWaypoint],
});
matchQuery.route = {
  driverDistance: 150120,
  driverDuration: 6540,
  passengerDistance: 150120,
  passengerDuration: 6540,
  fwdAzimuth: 276,
  backAzimuth: 96,
  points: [
    {
      lat: 48.689445,
      lon: 6.17651,
    },
    {
      lat: 48.7566,
      lon: 4.3522,
    },
    {
      lat: 48.8566,
      lon: 2.3522,
    },
  ],
};

const mockMatcherRepository: AdRepositoryPort = {
  insertExtra: jest.fn(),
  findOneById: jest.fn(),
  findOne: jest.fn(),
  insert: jest.fn(),
  update: jest.fn(),
  updateWhere: jest.fn(),
  delete: jest.fn(),
  count: jest.fn(),
  healthCheck: jest.fn(),
  getCandidates: jest.fn().mockImplementation(() => [
    {
      uuid: 'cc260669-1c6d-441f-80a5-19cd59afb777',
      driver: true,
      passenger: true,
      frequency: Frequency.PUNCTUAL,
      fromDate: new Date('2023-06-21'),
      toDate: new Date('2023-06-21'),
      seatsProposed: 3,
      seatsRequested: 1,
      strict: false,
      ddriverDistance: 350000,
      driverDuration: 14400,
      passengerDistance: 350000,
      passengerDuration: 14400,
      fwdAzimuth: 273,
      backAzimuth: 93,
      createdAt: new Date('2023-06-20T17:05:00Z'),
      updatedAt: new Date('2023-06-20T17:05:00Z'),
      waypoints: 'LINESTRING(6.1765102 48.689445,2.3522 48.8566)',
      schedule: [
        {
          day: 3,
          time: new Date('2023-06-21T07:05:00Z'),
          margin: 900,
        },
      ],
    },
  ]),
};

describe('Passenger oriented selector', () => {
  it('should select candidates', async () => {
    const passengerOrientedSelector: PassengerOrientedSelector =
      new PassengerOrientedSelector(matchQuery, mockMatcherRepository);
    const candidates: Candidate[] = await passengerOrientedSelector.select();
    expect(candidates.length).toBe(2);
  });
});
