import { AD_GET_BASIC_ROUTE_CONTROLLER } from '@modules/ad/ad.di-tokens';
import { Route } from '@modules/ad/core/application/types/route.type';
import { Role } from '@modules/ad/core/domain/ad.types';
import { RouteProvider } from '@modules/ad/infrastructure/route-provider';
import { GetBasicRouteControllerPort } from '@modules/geography/core/application/ports/get-basic-route-controller.port';
import { Test, TestingModule } from '@nestjs/testing';

const mockGetBasicRouteController: GetBasicRouteControllerPort = {
  get: jest.fn().mockImplementation(() => ({
    driverDistance: 23000,
    driverDuration: 900,
    passengerDistance: 23000,
    passengerDuration: 900,
    fwdAzimuth: 283,
    backAzimuth: 93,
    distanceAzimuth: 19840,
    points: [
      {
        lon: 6.1765103,
        lat: 48.689446,
      },
      {
        lon: 2.3523,
        lat: 48.8567,
      },
    ],
  })),
};

describe('Route provider', () => {
  let routeProvider: RouteProvider;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        RouteProvider,
        {
          provide: AD_GET_BASIC_ROUTE_CONTROLLER,
          useValue: mockGetBasicRouteController,
        },
      ],
    }).compile();

    routeProvider = module.get<RouteProvider>(RouteProvider);
  });

  it('should be defined', () => {
    expect(routeProvider).toBeDefined();
  });

  it('should provide a route', async () => {
    const route: Route = await routeProvider.getBasic(
      [Role.DRIVER],
      [
        {
          position: 0,
          lat: 48.689445,
          lon: 6.1765102,
        },
        {
          position: 1,
          lat: 48.8566,
          lon: 2.3522,
        },
      ],
    );
    expect(route.driverDistance).toBe(23000);
  });
});
